-- snowflake v1.2
-- by Amber (2022)

function setup()  
    viewer.mode = FULLSCREEN
    blendMode(ADDITIVE)
    lineCapMode(SQUARE)
    itt  = 4
    len  = math.min(WIDTH,HEIGHT)*0.15
    seed = math.random(math.maxinteger)
    smoothCircle = polarGradient(256)
    redraw()
end

function draw()
    background(0,0,0)
    translate(WIDTH/2,HEIGHT/2)   
    tint(255) ; sprite(screen)
end

function touched(touch)
    if touch.state == ENDED then seed = math.random(math.maxinteger) ; redraw() end --tap to reseed
end

function redraw()
    pushMatrix() ; resetMatrix()
    screen = image(WIDTH,HEIGHT) --clear screen
    setContext(screen)
    math.randomseed(seed) 
    translate(WIDTH/2,HEIGHT/2)   
    tint(32,0,96) ; sprite(smoothCircle,0,0,math.min(WIDTH,HEIGHT))
    tint(223, 192, 0) ; sprite(smoothCircle,0,0,math.min(WIDTH,HEIGHT)/6) 
    --draw 2 snowflakes, looks better!
    strokeWidth(len/9) ; stroke( 0,64,128) ; fill(0,32,64) ; snowflake(itt,   len*(5/6))
    strokeWidth(len/3) ; stroke(16,16,128) ; fill(8, 0,64) ; snowflake(itt-1, len      )
    setContext()
    popMatrix()
end

--snowflake functions
function snowflake(itt,len) 
    local ang = math.random(1,2)*60
    local offsetT = {} ; for i = 1,itt do offsetT[i] =  math.random() end --generate random offsets
    local lenghtT = {} ; for i = 1,itt do lenghtT[i] = (math.random()*0.5 +0.5)*len end --generate random lenghts
    --TODO: "normalize" the lenghtTable so "len" is guaranteed to be the radius of the snowflake..? 
    local function flakeSeg(itt,ang,offsetT,lenghtT)
        hexLine(0,0,0,lenghtT[itt]) ; translate(0,lenghtT[itt]*offsetT[itt])
        if itt > 1 then pushMatrix() ; rotate(-ang)
            for i = 1,3 do pushMatrix() ; flakeSeg(itt-1,ang,offsetT,lenghtT) ; popMatrix() ; rotate(ang) end popMatrix()
        end
    end--local function flakeSeg
    for i = 1,6 do flakeSeg(itt,ang,offsetT,lenghtT) ; translate(0,-lenghtT[itt]*offsetT[itt]) ; rotate(60) end
end

function hexLine(x,y,x2,y2)
    --get settings
    local extend ; if lineCapMode() ~= SQUARE then extend = true end
    local col1,col2 = color(stroke()), color(fill())
    local offset = strokeWidth()/2
    local d = (vec2(x,y)-vec2(x2,y2)):normalize()*offset -- direction
    local p -- points
    if extend then
        local l,r = d:rotate(math.rad(60)) , d:rotate(math.rad(-60))
        p = { vec2(x+l.x,y+l.y), vec2(x+d.x,y+d.y), vec2(x+r.x,y+r.y),vec2(x2-l.x,y2-l.y), vec2(x2-d.x,y2-d.y), vec2(x2-r.x,y2-r.y) }
    else
        local l,r = d:rotate(math.rad(120)) , d:rotate(math.rad(-120))
        p = { vec2(x+l.x,y+l.y), vec2(x,y), vec2(x+r.x,y+r.y), vec2(x2-l.x,y2-l.y), vec2(x2,y2), vec2(x2-r.x,y2-r.y) }
    end 
    local myHexLine = mesh()
    myHexLine.vertices = { p[1], p[2], p[5],   p[2], p[3], p[5],   p[3], p[4], p[5],   p[6], p[1], p[5] }
    myHexLine.colors =   { col1, col2, col2,   col2, col1, col2,   col1, col1, col2,   col1, col1, col2 }  
    myHexLine:draw()
    --pushStyle() ; fill(255) ;   for k,v in ipairs(p) do text(k,v.x,v.y) end   ; popStyle() --DEBUG: show point No.
end

--smooth gradient functions
function remappedSquared(x,y,x2,y2,radius)   
    local dSqr = (x-x2)^2 + (y-y2)^2 --vec2(x,y):distSqr(vec2(x2,y2))
    local n = math.max(0, 1-dSqr / radius^2 ) --normalized and clamped
    return 1-math.cos(math.pi*0.5*n)
end

function polarGradient(size,brightness)
    local brightness = brightness or 1 
    pushMatrix() ; resetMatrix()
    local img = image(size,size)
    local c
    for x = 1,size do
        for y = 1,size do
            c = math.min(remappedSquared(x,y,size*0.5,size*0.5,size*0.5)*brightness*255,255)
            img:set(x,y,color(c,c,c,255))
        end
    end
    popMatrix()
    return img
end